#include "opt_forwardreply.h"
#include "common.h"

#include <qwhatsthis.h>
#include <qcheckbox.h>

#include <qgroupbox.h>
#include <qtextedit.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qsyntaxhighlighter.h>
#include <qlistbox.h>

#include "opt_forwardreply-ui.h"

//----------------------------------------------------------------------------
// TemplatePopup
//----------------------------------------------------------------------------

class TemplatePopup : public QPopupMenu {
	Q_OBJECT
public:
	TemplatePopup(QTextEdit *edt, QWidget * parent = 0, const char * name = 0);
protected:
	int insertTemplateItem(QString name, QString tag, QString desc);
private:
	QTextEdit *edit;
	QMap<int, QString> tags;
private slots:	
	void onActivated(int id);
};

TemplatePopup::TemplatePopup(QTextEdit *edt, QWidget *parent, const char *name) : edit(edt), QPopupMenu(parent, name)
{
	connect(this, SIGNAL(activated(int)), this, SLOT(onActivated(int)));	
}

int TemplatePopup::insertTemplateItem(QString name, QString tag, QString desc="")
{
	int id = insertItem(name, this, SLOT(onActivated()));
	//setWhatsThis(id, "<b><i>" + tag + "</i></b><br>" + ((desc!="")?desc:name));
	tags[id] = tag;

	return id;
}

void TemplatePopup::onActivated(int id)
{
    if (edit) {
		edit->insert(tags[id]);
	}
}

//----------------------------------------------------------------------------
// QuoteHeaderPopup
//----------------------------------------------------------------------------

class QuoteHeaderPopup : public TemplatePopup {
	Q_OBJECT
public:
	QuoteHeaderPopup(QTextEdit *edt, QWidget *parent = 0, const char *name = 0);
};

QuoteHeaderPopup::QuoteHeaderPopup(QTextEdit *edt, QWidget * parent, const char * name)
	: TemplatePopup(edt, parent, name)
{
	insertTemplateItem("Sender's bare jid", "<Sender.jid.bare>");
	insertTemplateItem("Sender's full jid", "<Sender.jid.full>");
	insertTemplateItem("Sender's nick", "<Sender.nick>");
	insertTemplateItem("Sender's nick or jid", "<Sender.nick-jid>");
	insertSeparator();
	insertTemplateItem("Your bare jid", "<Me.jid.bare>");
	insertTemplateItem("Your full jid", "<Me.jid.full>");
	insertTemplateItem("Your nick", "<Me.nick>");
	insertTemplateItem("Your nick or jid", "<Me.nick-jid>");
	insertSeparator();
	insertTemplateItem("Message date and time", "<Message.DateTime>");
	insertTemplateItem("Message subject", "<Message.subject>");
	insertSeparator();
	insertTemplateItem("\'<\' character", "<>");
}

//----------------------------------------------------------------------------
// SignaturePopup
//----------------------------------------------------------------------------

class SignaturePopup : public TemplatePopup {
	Q_OBJECT
public:
	SignaturePopup(QTextEdit *edt, QWidget *parent = 0, const char *name = 0);
};

SignaturePopup::SignaturePopup(QTextEdit *edt, QWidget * parent, const char * name)
	: TemplatePopup(edt, parent, name)
{
	insertTemplateItem("Random text", "<RandomText>");
	insertTemplateItem("\'<\' character", "<>");
}

//----------------------------------------------------------------------------
// TemplateEditHighlighter
//----------------------------------------------------------------------------

class TemplateEditHighlighter : public QSyntaxHighlighter {
public:
	TemplateEditHighlighter(QTextEdit *edt) : QSyntaxHighlighter(edt) {}
	int highlightParagraph(const QString & text, int endStateOfLastPara);
protected:
	void addTag(QString tag);	// this should be used only when constructing the highlighter
								// in other case, rehighlighting of all text would be needed
private:
	QValueVector<QString> tags;
};

int TemplateEditHighlighter::highlightParagraph ( const QString & text, int /*endStateOfLastPara*/ )
{
	setFormat(0, text.length(), textEdit()->colorGroup().text());

	for ( unsigned i = 0; i < tags.size(); ++i ) {
		int first = 0;
		int start = -1;
		int len = tags[i].length();
		while ( (start = text.find(tags[i], first, false)) != -1 ) {
			setFormat(start, len, textEdit()->colorGroup().link());	// color of a link - it suggests something 'special' ;-)
																	// (and has good contrast)
			first = start + len + 1;
		}
	}
	return 0;
}

void TemplateEditHighlighter::addTag(QString tag)
{
	tags.push_back(tag);
}

//----------------------------------------------------------------------------
// QuoteHeaderHighlighter
//----------------------------------------------------------------------------

class QuoteHeaderHighlighter : public TemplateEditHighlighter {
public:
	QuoteHeaderHighlighter(QTextEdit *edt);
};

QuoteHeaderHighlighter::QuoteHeaderHighlighter(QTextEdit *edt) : TemplateEditHighlighter(edt)
{
	addTag("<Sender.jid.bare>");
	addTag("<Sender.jid.full>");
	addTag("<Sender.nick>");
	addTag("<Sender.nick-jid>");

	addTag("<Me.jid.bare>");
	addTag("<Me.jid.full>");
	addTag("<Me.nick>");
	addTag("<Me.nick-jid>");

	addTag("<Message.DateTime>");
	addTag("<Message.subject>");
	
	addTag("<>");	
}

//----------------------------------------------------------------------------
// SignatureHighlighter
//----------------------------------------------------------------------------

class SignatureHighlighter : public TemplateEditHighlighter {
public:
	SignatureHighlighter(QTextEdit *edt);
};

SignatureHighlighter::SignatureHighlighter(QTextEdit *edt) : TemplateEditHighlighter(edt)
{
	addTag("<RandomText>");
}

//----------------------------------------------------------------------------
// OptionsTabForwardReply
//----------------------------------------------------------------------------

OptionsTabForwardReply::OptionsTabForwardReply(QObject *parent)
: OptionsTab(parent, "forwardreply", "", tr("Replying"), tr("Advanced options for quoting and replying messages"), "psi/sendMessage")
{
	w = 0;
	//o = new Options;
}

OptionsTabForwardReply::~OptionsTabForwardReply()
{
	//delete o;
}

QWidget *OptionsTabForwardReply::widget()
{
	if ( w )
		return 0;

	w = new OptForwardReplyUI();
	
	OptForwardReplyUI *d = (OptForwardReplyUI *)w;

	// don't worry: these menus are deleted in OptForwardReplyUI::destroy()
	d->pb_insForward->setPopup(new QuoteHeaderPopup(d->te_headerForward));
	d->pb_insReply->setPopup(new QuoteHeaderPopup(d->te_headerReply));
	d->pb_insQuote->setPopup(new QuoteHeaderPopup(d->te_headerQuote));
	d->pb_insSignature->setPopup(new SignaturePopup(d->te_signature));
	
	// syntax highlighters are deleted by edit boxes
	new QuoteHeaderHighlighter(d->te_headerForward);
	new QuoteHeaderHighlighter(d->te_headerReply);
	new QuoteHeaderHighlighter(d->te_headerQuote);
	new SignatureHighlighter(d->te_signature);

	return w;
}

void OptionsTabForwardReply::applyOptions(Options *opt)
{
	if ( !w )
		return;

	OptForwardReplyUI *d = (OptForwardReplyUI *)w;

	opt->useQuoteHeaderForward = d->ck_headerForward->isChecked();
	opt->quoteHeaderForward = d->te_headerForward->text();
	opt->useQuoteHeaderQuote = d->ck_headerQuote->isChecked();
	opt->quoteHeaderQuote = d->te_headerQuote->text();
	opt->useQuoteHeaderReply = d->ck_headerReply->isChecked();
	opt->quoteHeaderReply = d->te_headerReply->text();

	opt->useMessageSignature = d->ck_signature->isChecked();
	opt->messageSignature = d->te_signature->text();
	opt->autoMessageSignatureDelimiter = d->ck_signatureAutoDelimiter->isChecked();

	//opt->randomSignatures = o->randomSignatures;

	opt->quoteWrapAtColumn = d->sb_wrapAtColumn->value();
	opt->quoteEmptyLines = d->ck_quoteEmptyLines->isChecked();
	opt->quoteStripEmptyLines = d->ck_stripEmptyLines->isChecked();
	opt->quoteRemoveSignature = d->ck_removeSignature->isChecked();
	opt->cursorAtTopInEventDlg = d->ck_cursorAtTop->isChecked();
}

void OptionsTabForwardReply::restoreOptions(const Options *opt)
{
	if ( !w )
		return;

	OptForwardReplyUI *d = (OptForwardReplyUI *)w;

	d->ck_headerForward->setChecked( true );
	d->ck_headerForward->setChecked( opt->useQuoteHeaderForward );
	d->te_headerForward->setText( opt->quoteHeaderForward );
	d->ck_headerQuote->setChecked( true );
	d->ck_headerQuote->setChecked( opt->useQuoteHeaderQuote );
	d->te_headerQuote->setText( opt->quoteHeaderQuote );
	d->ck_headerReply->setChecked( true );
	d->ck_headerReply->setChecked( opt->useQuoteHeaderReply );
	d->te_headerReply->setText( opt->quoteHeaderReply );

	d->ck_signature->setChecked( true );
	d->ck_signature->setChecked( opt->useMessageSignature );
	d->te_signature->setText( opt->messageSignature );
	d->ck_signatureAutoDelimiter->setChecked( opt->autoMessageSignatureDelimiter );

	//o->randomSignatures = opt->randomSignatures;

	d->sb_wrapAtColumn->setValue( opt->quoteWrapAtColumn );
	d->ck_quoteEmptyLines->setChecked( opt->quoteEmptyLines );
	d->ck_stripEmptyLines->setChecked( opt->quoteStripEmptyLines );
	d->ck_removeSignature->setChecked( opt->quoteRemoveSignature );
	d->ck_cursorAtTop->setChecked( opt->cursorAtTopInEventDlg );
}

#include "opt_forwardreply.moc"
